<?php

GFForms::include_feed_addon_framework();

class GFMiniCRM extends GFFeedAddOn {

  protected $_version = GF_MINICRM_VERSION;
  protected $_min_gravityforms_version = '1.9.14.26';
  protected $_slug = 'gravityformsminicrm';
  protected $_path = 'gravityformsminicrm/minicrm.php';
  protected $_full_path = __FILE__;
  protected $_url = 'http://www.gravityforms.com';
  protected $_title = 'Gravity Forms MiniCRM Add-On';
  protected $_short_title = 'MiniCRM';
  protected $api = null;
  protected $_new_custom_fields = array();

  /**
   * Members plugin integration
   */
  protected $_capabilities = array( 'gravityforms_minicrm', 'gravityforms_minicrm_uninstall' );

  /**
   * Permissions
   */
  protected $_capabilities_settings_page = 'gravityforms_minicrm';
  protected $_capabilities_form_settings = 'gravityforms_minicrm';
  protected $_capabilities_uninstall = 'gravityforms_minicrm_uninstall';
  protected $_enable_rg_autoupgrade = false;

  private static $_instance = null;

  private $httpClient;

  /**
   * Get an instance of this class.
   *
   * @return GFMiniCRM
   */
  public static function get_instance() {

    if ( self::$_instance == null ) {
      self::$_instance = new GFMiniCRM();
    }

    return self::$_instance;

  }

  public function __construct() {
    parent::__construct();

    if (!class_exists('\GuzzleHttp\Client')) {
      require_once 'vendor/autoload.php';
    }

    $this->httpClient = new \GuzzleHttp\Client();

    add_filter('gform_predefined_choices', array($this, 'predefined_choices'));

    $form = $this->get_current_form();
    if($form === false) return;

    $feeds = $this->get_feeds();
    $has_minicirm_feed = false;
    foreach ($feeds as $feed) {
      if ($feed['form_id'] != $form['id'] || empty($feed['meta']['minicrm_form_fields_cache'])) {
        $has_minicirm_feed = true;
        break;
      }
    }
    if ($has_minicirm_feed) {
      add_filter('gform_entry_field_value', array($this, 'entry_field_value'), 10, 4);
      add_filter('gform_entries_field_value', array($this, 'entries_field_value'), 10, 4);
      add_filter('gform_countries', array($this, 'gform_countries'));
    }
  }

  public function predefined_choices($choices) {
    $feeds = $this->get_feeds();
    $currentFormId = $this->get_current_form()['id'];

    foreach ($feeds as $feed) {
      if ($feed['form_id'] != $currentFormId || empty($feed['meta']['minicrm_form_fields_cache'])) {
        continue;
      }

      $minicrmFields = $feed['meta']['minicrm_form_fields_cache'];
      foreach ($minicrmFields as $field) {
        if (!empty($field['choices'])) {
          $preChoices = [];
          foreach ($field['choices'] as $choice) {
            $preChoices[] = "$choice[label]|$choice[value]";
          }

          $choices["<strong>MiniCRM</strong><br>Feed: ".$feed['meta']['feed_name']."<br>$field[legend] -> $field[label]"] = $preChoices;
        }
      }
    }

    return $choices;
  }

  public function entry_field_value($value, $field, $entry, $form) {
    $classes = array(
      'GF_Field_Checkbox',
      'GF_Field_MultiSelect',
      'GF_Field_Radio',
      'GF_Field_Select',
    );

    foreach ($classes as $class) {
      if ($field instanceof $class) {
        $value = $field->get_value_entry_detail(RGFormsModel::get_lead_field_value($entry, $field), $currency = '', $use_text = true, $format = 'html');
        break;
      }
    }

    return $value;
  }

  function entries_field_value($value, $form_id, $field_id, $entry) {
    $form         = GFAPI::get_form($form_id);
    $field        = RGFormsModel::get_field($form, $field_id);
    $value_fields = array(
      'checkbox',
      'radio',
      'select'
    );

    if (in_array($field->get_input_type(), $value_fields) ) {
      $value = $field->get_value_entry_detail(RGFormsModel::get_lead_field_value($entry, $field), '', true, 'text');
    }

    return $value;
  }

  public function gform_countries($countries = array()) {
    $currentFormId = $this->get_current_form()['id'];
    $feeds = $this->get_feeds();

    foreach ($feeds as $feed) {
      if ($feed['form_id'] != $currentFormId || empty($feed['meta']['minicrm_form_fields_cache'])) {
        continue;
      }

      $minicrmFields = $feed['meta']['minicrm_form_fields_cache'];
      foreach ($minicrmFields as $field) {
        if (strpos($field['name'], 'CountryId') > -1) {
          $countries = array();

          foreach ($field['choices'] as $choice) {
            $countries[intval($choice['value'])] = $choice['label'];
          }

          return $countries;
        }
      }
    }

    return $countries;
  }

  // # FEED PROCESSING -----------------------------------------------------------------------------------------------


  /**
   * Process the feed, subscribe the user to the list.
   *
   * @param array $feed The feed object to be processed.
   * @param array $entry The entry object currently being processed.
   * @param array $form The form object currently being processed.
   *
   * @return bool|void
   */
  public function process_feed( $feed, $entry, $form ) {

    $this->log_debug( __METHOD__ . '(): Processing feed.' );

    $minicrm_fields_encoded = $this->get_field_map_fields($feed, 'minicrm_fields');
    $minicrm_fields = [];
    foreach ($minicrm_fields_encoded as $key => $value) {
      $minicrm_fields[urldecode($key)] = $value;
    }

    $post = [
      'Referrer' => $_SERVER['HTTP_REFERER'],
      '__utma'   => $_COOKIE['__utma'] || '',
      '__utmz'   => $_COOKIE['__utmz'] || '',
      'FormHash' => $this->get_minicrm_form_hash($feed['meta']['form_code'])
    ];
    foreach ($minicrm_fields as $key => $fieldId) {
      $value = $this->get_minicrm_field_value($form, $entry, $fieldId, $key);
      if (!empty($value)) {
        if (strpos($value, ', ') > 0 && substr($key, -2) === '[]') {
          $post[substr($key, 0, strlen($key) - 2)] = explode(', ', $value);
        } else {
          $post[$key] = $value;
        }

        if (strpos($key, 'CountryId') > -1 && !is_numeric($value)) {
          $countries = array_flip($this->gform_countries());
          $post[$key] = $countries[$value];
        }
      }
    }

    $multipart = [];
    foreach ($post as $key => $value) {
      if (strpos($key, 'File') > 0) {
        $values = explode(' , ', $value);
        foreach ($values as $key2 => $value2) {
          $upload_path = GFFormsModel::get_upload_path($entry['form_id']);
          $upload_url = GFFormsModel::get_upload_url($entry['form_id']);
          $filepath = str_replace('/', DIRECTORY_SEPARATOR, str_replace($upload_url, $upload_path, $value2));

          $multipart[] = [
            'name' => $key . "[$key2]",
            'contents' => fopen($filepath, 'r'),
            'filename' => basename($filepath)
          ];
        }
      } else if (is_array($value)) {
        foreach ($value as $k => $v) {
          $multipart[] = [
            'name' => $key."[$k]",
            'contents' => $v
          ];
        }
      } else {
        $multipart[] = [
          'name' => $key,
          'contents' => $value
        ];
      }
    }

    try {
      $resp = $this->httpClient->request('POST', 'https://r3.minicrm.hu/Api/Signup', [ 'multipart' => $multipart ]);
      $body = (string) $resp->getBody();
      $bodyDecoded = json_decode($body, true);

      if (!json_last_error()) {
        if (!empty($bodyDecoded['Error'])) {
          $errorsMsg = $bodyDecoded['Error'];
          $this->add_feed_error(sprintf(esc_html__('Received error from MiniCRM: %s', 'gravityformsminicrm'), $errorsMsg), $feed, $entry, $form);
          return;
        }
      }
    } catch (Exception $e) {
      $this->add_feed_error(sprintf(esc_html__('Something went wrong with the request sent to MiniCRM: %s', 'gravityformsminicrm'), $e->getMessage()), $feed, $entry, $form);
			return;
    }
  }

  // # ADMIN FUNCTIONS -----------------------------------------------------------------------------------------------

  /**
   * Plugin starting point. Handles hooks, loading of language files and PayPal delayed payment support.
   */
  public function init() {

    parent::init();

    $this->add_delayed_payment_support(
      array(
        'option_label' => esc_html__( 'Subscribe contact to MiniCRM only when payment is received.', 'gravityforms' )
      )
    );

  }

  // ------- Plugin settings -------

  /**
   * Prepare plugin settings description.
   *
   * @return string
   */
  public function plugin_settings_description() {

    return 'Integrates Gravity Forms with MiniCRM allowing form submissions to be automatically sent to your MiniCRM account.';

  }

  // ------- Feed page -------

  /**
   * Prevent feeds being listed or created if the api key isn't valid.
   *
   * @return bool
   */
  public function can_create_feed() {

    return true;

  }

  /**
   * Enable feed duplication.
   *
   * @access public
   * @return bool
   */
  public function can_duplicate_feed( $id ) {

    return true;

  }

  /**
   * Configures which columns should be displayed on the feed list page.
   *
   * @return array
   */
  public function feed_list_columns() {
    return array(
      'feed_name'  => esc_html__( 'Name', 'gravityformsminicrm' )
    );
  }

  /**
   * Configures the settings which should be rendered on the feed edit page.
   *
   * @return array The feed settings.
   */
  public function feed_settings_fields() {
    $feed = $this->get_posted_settings();
    if (empty($feed)) {
      $feed = $this->get_current_feed()['meta'];
    }

    $fields = array(
      array(
        'name'     => 'feed_name',
        'label'    => esc_html__( 'Name', 'gravityforms' ),
        'type'     => 'text',
        'required' => true,
        'tooltip'  => '<h6>' . esc_html__( 'Name', 'gravityforms' ) . '</h6>' . esc_html__( 'Enter a feed name to uniquely identify this setup.', 'gravityforms' )
      ),
      array(
        'name'     => 'form_code',
        'label'    => esc_html__( 'MiniCRM Form Code', 'gravityformsminicrm' ),
        'type'     => 'textarea',
        'rows'     => 4,
        'cols'     => 100,
        'tooltip'  => '<h6>' . esc_html__( 'MiniCRM Form Code', 'gravityforms' ) . '</h6>' . esc_html__( 'Paste your MiniCRM\'s form embed code here.', 'gravityformsminicrm' ),
        'required' => true,
        'validation_callback' => array($this, 'validate_form_code'),
      )
    );

    $fields[] = array(
      'name'       => 'minicrm_fields',
      'label'      => esc_html__( 'Map Fields', 'gravityformsminicrm' ),
      'type'       => 'minicrm_field_map',
      'dependency' => 'form_code',
      'field_map'  => $this->minicrm_fields_for_feed_mapping($feed),
      'tooltip'    => '<h6>' . esc_html__( 'Map Fields', 'gravityformsminicrm' ) . '</h6>' . esc_html__( 'Select which MiniCRM fields pair with their respective GravityForms field.', 'gravityformsminicrm' )
    );

    /* Add feed condition and options fields. */
    $fields[] = array(
      'name'       => 'feed_condition',
      'label'      => esc_html__( 'Conditional Logic', 'gravityforms' ),
      'type'       => 'feed_condition',
      'tooltip'    => '<h6>' . esc_html__( 'Conditional Logic', 'gravityforms' ) . '</h6>' . esc_html__( 'When conditional logic is enabled, form submissions will only be exported to MiniCRM when the condition is met. When disabled, all form submissions will be exported.', 'gravityforms' )
    );

    return array(
      array(
        'title'  => '',
        'fields' => $fields
      )
    );

  }

  public function validate_form_code($field, $value) {
    if (!$this->get_minicrm_form_hash($value)) {
      $this->set_field_error( array( 'name' => 'form_code' ), esc_html__( 'Failed to identify a MiniCRM form from the given form code.', 'gravityformsminicrm' ) );
      return;
    }
  }

  public function save_feed_settings($feed_id, $form_id, $settings) {
    $settings['minicrm_form_hash_cache'] = $this->get_minicrm_form_hash($settings['form_code']);
    $settings['minicrm_form_url_cache'] = $this->get_minicrm_form_url($settings['minicrm_form_hash_cache']);
    $settings['minicrm_form_fields_cache']  = $this->get_minicrm_form_fields($settings['minicrm_form_url_cache']);

    return parent::save_feed_settings($feed_id, $form_id, $settings);
  }

  // ------- Helper functions -------

  public function get_minicrm_form_hash($formCode) {
    preg_match('/<script src=\'https:\/\/r3\.minicrm\.hu\/api\/loader\.js\?(.*)\'><\/script>/', $formCode, $matches);

    if (count($matches) < 2) {
      return false;
    }

    return $matches[1];
  }

  public function get_minicrm_form_url($formHash) {
    return "https://r3.minicrm.hu/Form-$formHash";
  }

  public function get_minicrm_form_fields($formUrl) {
    $fields = [];

    $html = $this->httpClient->request('GET', $formUrl)->getBody();
    $dom = new DOMDocument();
    $dom->loadHTML('<?xml encoding="utf-8" ?>' . $html);

    $xpath = new DOMXPath($dom);
    $fieldsets = $xpath->query("//fieldset");
    foreach ($fieldsets as $fieldset) {
      $legend = $xpath->query("./legend", $fieldset);
      if ($legend->length > 0) {
        $legend = $legend[0];

        $inputBlocks = $xpath->query(".//div[@class='InputBlock']", $fieldset);
        foreach ($inputBlocks as $inputBlock) {
          $labels = $xpath->query(".//label", $inputBlock);
          $inputs = $xpath->query(".//*[local-name() = 'input' or local-name() = 'textarea' or local-name() = 'select']", $inputBlock);

          if ($labels->length > 0 && $inputs->length > 0) {
            $name = $inputs[0]->getAttribute('name');
            $labelText = $labels[0]->textContent;

            $nodeName = strtolower($inputs[0]->nodeName);
            $type = null;
            switch ($nodeName) {
              case 'textarea':
              case 'select':
                $type = $nodeName;
                break;
              case 'input':
                $inputType = strtolower($inputs[0]->getAttribute('type'));
                $type = $inputType;
                break;
            }

            $choices = [];
            if ($type === 'select') {
              $options = $xpath->query(".//option", $inputs[0]);

              foreach ($options as $option) {
                $label = $option->textContent;
                $value = $option->getAttribute('value');
                if (!empty($label) && !empty($value)) {
                  $choices[] = compact('label', 'value');
                }
              }
            } else if ($type === 'checkbox' || $type === 'radio') {
              $spans = $xpath->query(".//span", $inputBlock);
              foreach ($inputs as $i => $input) {
                $label = $spans[$i]->textContent;
                $value = $input->getAttribute('value');
                if (!empty($label) && !empty($value)) {
                  $choices[] = compact('label', 'value');
                }
              }
            }

            $fields[] = array(
              'legend'  => trim($legend->textContent),
              'label'   => trim($labelText),
              'name'    => trim($name),
              'type'    => $type,
              'choices' => $choices
            );
          }
        }
      }
    }

    return $fields;
  }

  public function minicrm_fields_for_feed_mapping($feed) {
    $formHash = $this->get_minicrm_form_hash($feed['form_code']);
    $formUrl = $this->get_minicrm_form_url($formHash);
    $minicrmFields = $this->get_minicrm_form_fields($formUrl);

    $fields = [];
    foreach ($minicrmFields as $minicrmField) {
      $fieldType = null;
      $defaultValue = null;

      if ($minicrmField['type'] === 'file') {
        $fieldType = array('fileupload');
        $defaultValue = $this->get_first_field_by_type($fieldType[0]);
      }

      $fields[] = array(
        'name'          => urlencode($minicrmField['name']),
        'label'         => $minicrmField['legend'] . ' -> '. $minicrmField['label'],
        'field_type'    => $fieldType,
        'default_value' => $defaultValue
      );
    }

    return $fields;
  }

  /* https://github.com/wp-premium/gravityforms/blob/e2196f015b0d0a0f28367ff0fedcbe4b3967eca3/includes/addon/class-gf-addon.php#L2303 */
  public function settings_minicrm_field_map( $field, $echo = true ) {
    // Initialize return HTML string.
    $html = '';
    // Get field map choices.
    $field_map = rgar( $field, 'field_map' );
    // If no field map choices exist, return HTML.
    if ( empty( $field_map ) ) {
      return $html;
    }
    // Get current form ID.
    $form_id = rgget( 'id' );
    // Display field map table header.
    $html .= '<table class="settings-field-map-table" cellspacing="0" cellpadding="0">' .
            $this->field_map_table_header() .
                  '<tbody>';
    // Loop through field map choices.
    foreach ( $field['field_map'] as $child_field ) {
      // If field map choice does not meet the dependencies required to be displayed, skip it.
      if ( ! $this->setting_dependency_met( rgar( $child_field, 'dependency' ) ) ) {
        continue;
      }
      // Get field map choice name, tooltip and required indicator.
      $child_field['name'] = $this->get_mapped_field_name( $field, $child_field['name'] );
      $tooltip             = $this->maybe_get_tooltip( $child_field );
      $required            = rgar( $child_field, 'required' ) ? ' ' . $this->get_required_indicator( $child_field ) : '';

      if (strpos($child_field['name'], 'DateTime') !== FALSE) {
        $child_field2 = $child_field;

        $child_field['name'] .= '[]';
        $child_field2['name'] .= '[]';

        $child_field['field_type'] = array('date', 'text', 'hidden');
        $child_field2['field_type'] = array('time', 'text', 'hidden');

        $child_field['default_value'] = $this->get_first_field_by_type($child_field['field_type'][0]);
        $child_field2['default_value'] = $this->get_first_field_by_type($child_field2['field_type'][0]);

        $childFieldHtml = $this->settings_field_map_select( $child_field, $form_id );
        $childFieldHtml .= $this->settings_field_map_select( $child_field2, $form_id );
      } else {
        $childFieldHtml = $this->settings_field_map_select( $child_field, $form_id );
      }

      // Display field map choice row.
      $html .= '
                <tr>
                    <td>
                        <label for="' . $child_field['name'] . '">' . $child_field['label'] . $tooltip . $required . '</label>
                    </td>
                    <td>
                      ' . $childFieldHtml  . '
                    </td>
            </tr>';
    }
    // Close field map table.
    $html .= '
                </tbody>
            </table>';
    // If automatic display is enabled, echo field HTML.
    if ( $echo ) {
      echo $html;
    }
    return $html;
  }

  public function get_minicrm_field_value($form, $entry, $fieldId, $key) {
    if (is_array($fieldId)) {
      if (strpos($key, 'DateTime') !== FALSE) {
        $date = $this->get_field_value($form, $entry, $fieldId[0]);
        $time = $this->get_field_value($form, $entry, $fieldId[1]);
        if ($date && $time) {
          $parsed = strtotime($date . ' ' . $time);
          if ($parsed !== FALSE) {
            return date('Y-m-d H:i:s', $parsed);
          }
        } else if ($date) {
          $parsed = strtotime($date);
          if ($parsed !== FALSE) {
            return date('Y-m-d', $parsed);
          }
        }
      }

      return '';
    }

    return $this->get_field_value($form, $entry, $fieldId);
  }

}